#
# Project Model for DBSE-2006-Task-OSS-A
#
__Version__ = "0.1.4"
__Owner__ = "seLain"
__Update_Date__ = "2006/09/19"


class OSSAProject:

    def __init__(self):
        self.packages = []           #default

    def number_of_class(self):
        accClasses = 0
        for pkg in self.packages:
            accClasses = accClasses + pkg.number_of_class()
        return accClasses

    def number_of_inheritence(self):
        accInheritence = 0
        for pkg in self.packages:
            accInheritence = accInheritence + pkg.number_of_inheritence()
        return accInheritence

    def number_of_internalBaseClass(self):
        accInternalBaseClass = 0
        for pkg in self.packages:
            accInternalBaseClass = accInternalBaseClass + \
                                   pkg.number_of_internalBaseClass()
        return accInternalBaseClass


class OSSAPackage:

    def __init__(self):
        # classes is a dictionary with form:
        #   {key:value,} = {class-fullname:class-object,}
        self.classes = {}           #default

    def number_of_class(self):
        return len(self.classes.keys())

    def number_of_internalBaseClass(self):
        baseClassesNames = []
        for key in self.classes.keys():
            aClass = self.classes[key]
            for parent in aClass.parents:
                # because there might be classes with same name
                # among different namespace, so using fullname here
                if parent not in baseClassesNames:
                    baseClassesNames.append(parent)
        return len(baseClassesNames)

    def number_of_inheritence(self):
        accInheritence = 0
        for key in self.classes.keys():
            aClass = self.classes[key]
            accInheritence = accInheritence + len(aClass.parents)
        return accInheritence

    '''
    counts for inherited operations in a class, including directly and
    inderectly inherited opetations
    @param className : name of target class
    '''
    def number_of_inherited_operation_in_class(self, className):

        def recursive_search_inherited_operations(self, className):
            recursiveInheritence = []
            # [!!] due to library usage, some parent class may not exist
            # in current package...
            if className in self.classes.keys():
                targetClass = self.classes[className]
                for parent in targetClass.parents:
                    recursiveInheritence = recursiveInheritence + \
                                           recursive_search_inherited_operations(self, parent)
                recursiveInheritence = recursiveInheritence + \
                                       targetClass.inheritable_operations()
            return recursiveInheritence
                                   
        allInheritedOperations = []
        targetClass = self.classes[className]
        for parent in targetClass.parents:
            allInheritedOperations = allInheritedOperations + \
                                     recursive_search_inherited_operations(self, parent)
        return len(allInheritedOperations)
            
        

class OSSAClass:

    def __init__(self):
        self.name = "name-default"      #default
        # fullname : name with namespace prefix
        self.fullname = "full-default"  #default
        self.parents = []               #default
        self.childs = []                #default
        self.operations = []            #default
        self.attributes = []            #default
        self.isAbstract = False         #default
 
    def number_of_private_attribute(self):
        accPriAttr = 0
        for attribute in self.attributes:
            if attribute.visibility == 'private':
                accPriAttr = accPriAttr + 1
        return accPriAttr

    def number_of_attribute(self):
        return len(self.attributes)

    def number_of_operation(self):
        return len(self.operations)

    def inheritable_operations(self):
        inheriOperations = []
        for operation in self.operations:
            if operation.is_inheritable() == True:
                inheriOperations.append(operation)
        return inheriOperations


class OSSAAttribute:

     def __init__(self):
        self.name = "name-default"          #default
        self.type = "type-default"          #default
        self.visibility = "public-default"  #default


class OSSAOperation:

    def __init__(self):
        self.name = "name-default"
        self.visibility = "public-default"  #default
        self.isVirtual = False              #default
        self.isAbstract = False            # default
        # owner is a class information object
        self.owner = None                   #default
        self.parameters = []                #default

    def number_of_parameter(self):
        return len(self.parameters)

    def is_inheritable(self):
        # [!!] attention here, what kind of operation is inheritable ?
        #      as long as "public" and "protected" ? what about "virtual" ?
        if self.visibility == "public" or self.visibility == "protected":
            return True
        else:
            return False

class OSSAParameter:

    def __init__(self):
        self.name = "name-default"          #default
        self.type = "type-default"          #default


#
# for unit testing
#
if __name__ == '__main__':

    # testing
    pass

    
    
