#
# Utils Module for DBSE-2006-Task-OSS-A
#
__Version__ = "0.1"
__Owner__ = "seLain"
__Update_Date__ = "2006/09/22"

'''
readCSV : read a csv file, and transform it into python dictioanry objects in a dictionary
@param filename : the csv file name, with full name, ex. "grade.csv"
@document :
	the first row and the first column are used as keys and objects,
	for example, a csv file cotent like  :
	
	"column1", "column2", "column3", "column4"
	"obj1", 4.56, 34.2, 32.3
	"obj2", 6.26, 4.2, 8.3
	
	will be read an return as a dictionary of dictionaries :
	
	return_dict = {	"obj1":{"column2":4.56, "column3":34.2, "column4":32.3}, 
				"obj2":{"column2":6.26, "column3":4.2, "column4":8.3}	}
				
	notice that the "column1" will be ignored
'''
def readCSV(filename):
	dictCSV = {}
	csvFile = file(filename, "r")
	## processing first line
	allLines = csvFile.readlines()
	firstline = allLines[0]
	keyList = firstline.strip().split(",")[1:]	# ignore column 1
	## processing rest of the lines
	for line in allLines[1:]:
		contentList = line.strip().split(",")
		objName = contentList[0]
		valueList = contentList[1:]
		dictCSV[objName] = {}
		for key_index in range(len(keyList)):
			dictCSV[objName][keyList[key_index]] = valueList[key_index]
	return dictCSV


'''
writeCSV : write a python dictioanry objects into a csv file
@param filename : the csv file name, with full name, ex. "grade.csv"
@document :
	the first row and the first column are used as keys and objects,
	for example, a dictionary cotent like  :
	
	write_dict = {	"obj1":{"column2":4.56, "column3":34.2, "column4":32.3}, 
				"obj2":{"column2":6.26, "column3":4.2, "column4":8.3}	}
	
	will be write as csv file cotent like :
	
	"column1", "column2", "column3", "column4"
	"obj1", 4.56, 34.2, 32.3
	"obj2", 6.26, 4.2, 8.3
		
	notice that the "column1" will be ignored as empty
'''
def writeCSV(dictCSV, filename):
	csvFile = file(filename, "w")
	## write column
	columnList = dictCSV[dictCSV.keys()[0]].keys()
	csvFile.writelines(','+','.join(columnList) +'\n')
	## write content
	for key in dictCSV.keys():
		content =dictCSV[key]
		print content
		csvFile.writelines(key+',' +','.join(content.values())+'\n')


def normalize_table_in_CSV_file(rawTitle, csvFileName, normalizedFileName):
	
	dictCSV = readCSV(csvFileName)
	## get the "baseline" record from a copy of specific raw
	import copy
	baselineRecord = copy.deepcopy(dictCSV[rawTitle])
	## normalize the whole table
	for eachRawTitle in dictCSV.keys():
		currentRaw = dictCSV[eachRawTitle]
		for itemName in currentRaw.keys():
			# that baseline number should be an absolute value
			currentRaw[itemName] = str( float(currentRaw[itemName]) / abs(float(baselineRecord[itemName])) )
	## write into output csv file
	writeCSV(dictCSV, normalizedFileName)


############
## for testing ##
############
# writeCSV(readCSV("DBSE-Task-OSSA-MFC-ver1-to-Ver5-add-Composition.csv"), "test.csv")
# normalize_table_in_CSV_file("projects/wtl75_4196","output-QA.csv","output-QA-normalized.csv")
