#
# dotmodulizer reads a mdg.dot output file from Bunch Tool, and copy the sources in srcpath
# into different clustering firectory based on input mdg.dot file
#
__Version__ = "0.1"
__Owner__ = "seLain"
__Update_Date__ = "2006/12/23"

import os, re

###############################################################
## From : File Selection Wrapper from ASPN, 
##                  URL:http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/436507
## Author : Jarrod Roberson
###############################################################
class Action:
    """
    Actions are things that can be executed.
    """
        
    def execute(self):
        pass

class PatternSet:
    """
    Pattern set provides the interface and abstract functionality to provide the include and exclude
    semantics for any classes that want to filter their results based on include and exclude criteria
    """

    def __init__(self):
        self.includeList = []
        self.excludeList = []
        
    def include(self, pattern):
        """
        Patterns should only defined on a single target to match so that we can use the fast fail
        functionality when matching. If you need multiple patterns, call .include() multiple
        times with each expression
        """
        self.includeList.append(re.compile(pattern))
    
    def __isIncluded(self, name):
        result = False
        if len(self.includeList) > 0:
            for pattern in self.includeList:
                name = name.split('\\')[-1]     # [seLain:] modified by selain, attention, it may cause some problem 
                                                            #              when same filename occurs in diferent directory
                if pattern.match(name) != None:
                    result = True
                    break
        return result 

    def exclude(self, pattern):
        """
        Patterns should only defined on single target to match so that we can use the fast fail
        functionality when matching. If you need multiple patterns, call .include() multiple
        times with each expression
        """
        self.excludeList.append(re.compile(pattern))
    
    def __isNotExcluded(self,name):
        result = True
        if len(self.excludeList) > 0:
            for pattern in self.excludeList:
                if pattern.match(name) != None:
                    result = False
                    break
        return result
    
    def clear(self):
        self.includeList = []
        self.excludeList = []

class FileSet(PatternSet, Action):
    """
    This class allows the user to define a set of files to work on using simple include/exclude semantics
    coupled with the full power of regular expressions
    """    
    def __init__(self, rootDir):
        PatternSet.__init__(self)
        self.rootDir = rootDir

    def execute(self):
        """
        This implementation walks the filesystem from the rootDir and creates a list of
        fully qualified files and returns them.
        """
        results = []
        # walk the dirs from the rootDir and build the results
        for root, dirs, files in os.walk(self.rootDir):
            # if the name is not on the include list or is on the exclude
            # list remove it from the results
            for name in files:
                fqname = os.path.join(root,name)
                if self._PatternSet__isIncluded(fqname) and self._PatternSet__isNotExcluded(fqname):
                    results.append(fqname)
        return results
        
#################################################################

class SubGraphCluster:
    
    def __init__(self, id, label):
        self.clusterId = id
        self.label = label
        self.filenames = []
        self.resultSet = []


def mdg_dot_based_source_modulize(filename, srcpath):
    
    mdgdotFile = open(filename, 'r')
    lines = mdgdotFile.readlines()
    clusterFlags = []
    clusterOfLines = {}
    
    ## finding subgraph starting and ending flags
    accNum = 0
    for aLine in lines :
        if (aLine.find('subgraph') is not -1) or (aLine.find('}') is not -1) :
            clusterFlags.append(accNum)
        accNum = accNum + 1
    
    ## decompose lines into clusters of lines
    for index in range(len(clusterFlags)/2):
            clusterOfLines[index] = lines[clusterFlags[2*index]:clusterFlags[2*index+1]+1]

    ## parsing each cluster of lines
    allSubGraphClusters = []
    for key in clusterOfLines.keys():
        clusterLines = clusterOfLines[key]
        subgraphCluster = SubGraphCluster(key, clusterLines[1].split('"')[1])
        for index in range(5, len(clusterLines)):
            aLine = clusterLines[index]
            if aLine.find('"') is not -1:
                subgraphCluster.filenames.append(aLine.split('"')[1])
        allSubGraphClusters.append(subgraphCluster)
    
    ## find files and devide the source package directory into modulized directory
    fileSet = FileSet(srcpath)
    for subgraphCluster in allSubGraphClusters:
        fileSet.clear()
        for filename in  subgraphCluster.filenames:
            fileSet.include(filename)
        subgraphCluster.resultSet = fileSet.execute()
    
    import shutil
    for subgraphCluster in allSubGraphClusters:
        for filename in subgraphCluster.resultSet:
            distinationFilePath = os.path.join(os.path.join(os.getcwd(),
                                            filename.split('\\')[0].split('-')[0] + '_cluster_' + 
                                            str(subgraphCluster.clusterId) + '-' +
                                            filename.split('\\')[0].split('-')[1]),  
                                            filename)
            directoryPath = '\\'.join(distinationFilePath.split('\\')[:-1])
            if os.path.isdir(directoryPath) is False:
                os.makedirs(directoryPath)
            shutil.copyfile(filename, distinationFilePath)
    
    
    
if __name__ == '__main__':
    
    mdg_dot_based_source_modulize('FileZilla-2.2.29.mdg.dot', 'FileZilla-2.2.29')
