#
# Extrator for DBSE-2006-Task-OSS-A
#
__Version__ = "0.11"
__Owner__ = "seLain"
__Update_Date__ = "2006/09/03"

class DoxygenXMLExtractor:

    def __init__(self):
        pass

    '''
    Extracts all source information from a project directory
    @param projPath : target project directory path
    '''
    def extract_project(self, projPath):

        '''
        Extract operation information object and append to class information object
        @param section : XML section contains operations
        @param classObj : class object
        '''
        def extract_operations(section, classObj):
            allMemberdefs = section.findall('memberdef')
            for memberdef in allMemberdefs:
                if memberdef.attrib['kind'] == 'function':
                    operationObj = OOSAModel.OSSAOperation()
                    # extract name
                    operationObj.name = memberdef.find('name').text
                    # extract visibility
                    operationObj.visibility = memberdef.attrib['prot']
                    # extract virtual information
                    if memberdef.attrib['virt'] == 'virtual' or \
                        memberdef.attrib['virt'] == 'pure-virtual':
                        operationObj.isVirtual = True
                    else:
                        operationObj.isVirtual = False
                    # define owner
                    operationObj.owner = classObj
                    # extract parameters
                    allParams = memberdef.findall('param')
                    for param in allParams:
                        paramObj = OOSAModel.OSSAParameter()
                        # extract param name
                        if param.find('declname') is not None : 
                            paramObj.name = param.find('declname').text
                        #else:
                        #    paramObj.name = param.find('declname').text
                        # extract param type
                        # [!!] attention, it's dirty here, too many exceptions here...:(
                        paramType = param.find('type')
                        if paramType.find('ref') is not None :
                            paramObj.type = paramType.find('ref').text
                        else:
                            paramObj.type = paramType.text
                        # append parameter into operation object
                        operationObj.parameters.append(paramObj)
                    # append operation into class object
                    classObj.operations.append(operationObj)
                    
        '''
        Extract operation attribute object and append to class information object
        @param section : XML section contains attributes
        @param classObj : class object
        '''
        def extract_attributes(section, classObj):
            allMemberdefs = section.findall('memberdef')
            for memberdef in allMemberdefs:
                if memberdef.attrib['kind'] == 'variable':
                    attributeObj = OOSAModel.OSSAAttribute()
                    # extract name
                    attributeObj.name = memberdef.find('name').text
                    # extract type
                    if memberdef.find('type').find('ref') is not None:
                        # a very dirty here :(
                        # too many exception situations...
                        if memberdef.find('type').text is not None:
                            attributeObj.type = memberdef.find('type').text.strip() + \
                                                memberdef.find('type').find('ref').text + '>'
                        else:
                            attributeObj.type = memberdef.find('type').find('ref').text
                    else:
                        attributeObj.type = memberdef.find('type').text
                    # extract visibility
                    attributeObj.visibility = memberdef.attrib['prot']
                    # append attribute into class object
                    classObj.attributes.append(attributeObj)
                    # for test
                    '''
                    print 'class name:', classObj.fullname
                    print 'attr name:', attributeObj.name
                    print 'attr type:', attributeObj.type
                    print 'attr visi:', attributeObj.visibility
                    '''

                    
        # get all filenames
        import os
        if not os.path.isdir(projPath):
            print 'Ooops, it\'s not a directory.'
            return
        listAllFilenames = os.listdir(projPath)
        fileToBeRemove = []
        # find out files that should be removed
        for filename in listAllFilenames:
            if filename.endswith('.xsd') or filename.endswith('.xslt'):
                fileToBeRemove.append(filename)
        # remove schema(.xsd) and XSLT(.xslt) files
        for filename in fileToBeRemove:
            listAllFilenames.remove(filename)

        # extract as OSSA information objects
        import OOSAModel
        projectObj = OOSAModel.OSSAProject()
        packageObj = OOSAModel.OSSAPackage()
        projectObj.packages.append(packageObj)

        # extract class information object
        for filename in listAllFilenames:
            xmlFile = open(projPath + '/' + filename, 'r')
            from elementtree.ElementTree import parse
            try:
                tree = parse(xmlFile)
            except:
                logFile = open("ErrorLog.txt","a")
                logFile.writelines(['parsing '+str(xmlFile)+' error\n'])
                continue
            elemRoot = tree.getroot()
            allCompounddef = elemRoot.findall('compounddef')
            for compounddef in allCompounddef:
                if compounddef.attrib['kind'] == 'class':
                    classObj = OOSAModel.OSSAClass()
                    # extract name
                    classObj.fullname = compounddef.find('compoundname').text
                    classObj.name = classObj.fullname.split('::')[-1]
                    # extract parents names(fullname)
                    baseCompounddefs = compounddef.findall('basecompoundref')
                    for basecompounddef in baseCompounddefs:
                        classObj.parents.append(basecompounddef.text)
                    # extract childs names(fullname)
                    derivedCompounddefs = compounddef.findall('derivedcompoundref')
                    for derivedcompounddef in derivedCompounddefs:
                        classObj.childs.append(derivedcompounddef.text)
                    # get sections
                    allSections = compounddef.findall('sectiondef')
                    for section in allSections:
                        # extract operation information object
                        if section.attrib['kind'].find('func') is not -1 :
                            extract_operations(section, classObj)
                        # extract attribute information object    
                        elif section.attrib['kind'].find('attrib') is not -1 :
                            extract_attributes(section, classObj)
                    '''
                    for section in allSections:
                        # extract operation information object
                        if section.attrib['kind'] == 'public-func' or \
                           section.attrib['kind'] == 'private-func' or \
                           section.attrib['kind'] == 'protected-func' or \
                           section.attrib['kind'] == 'package-func' :
                            extract_operations(section, classObj)
                        # extract attribute information object    
                        elif section.attrib['kind'] == 'public-attrib' or \
                             section.attrib['kind'] == 'private-attrib' or \
                             section.attrib['kind'] == 'protected-attrib'or \
                             section.attrib['kind'] == 'package-attrib' :
                            extract_attributes(section, classObj)
                    '''
                    # add class information object into package information object
                    if classObj.fullname not in packageObj.classes.keys():
                        packageObj.classes[classObj.fullname] = classObj

        return projectObj   # end of extract_proj(self, projPath)
                                   


        
#
# for unit testing
#
if __name__ == '__main__':

	extractor = DoxygenXMLExtractor()
	projectObj = extractor.extract_proj('xml')
	
	# testing for number_of_inherited_operation_in_class
	print '##testing for number_of_inherited_operation_in_class##'
	className = 'CAMSEdit::SelectionSaver'
	for packageObj in projectObj.packages:
		if className in packageObj.classes.keys():
			print 'num of inheri. operations of', className, '= ', \
			packageObj.number_of_inherited_operation_in_class(className)
	print ''

'''
    # testing for class numbe
    print '##testing for class number##'
    print 'number of classes=', projectObj.number_of_class()
    print ''

    # testing number of inheritence
    print '##testing number of inheritence##'
    print 'number of inheritence=', projectObj.number_of_inheritence()
    print ''
    
    # testing number of internal base classes
    print '##testing number of internal base classes##'
    print 'number of internal base class=', projectObj.number_of_internalBaseClass()
    print ''
    
    # testing number of private attribute
    print '##testing number of private attribute##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            print 'number of private attribute of', className, '=', \
                  classObj.number_of_private_attribute()
    print ''
    
    # testing number of attribute
    print '##testing number of attribute##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            print 'number of attribute of', className, '=', classObj.number_of_attribute()
    print ''
    
    # testing for attribute type
    print '##testing for attribute type##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            for attributeObj in classObj.attributes:
                print 'type of attribute', attributeObj.name,'of', className, '=', attributeObj.type
    print ''
    
    # testing for parameter type
    print '##testing for parameter type##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            for operationObj in classObj.operations:
                for paramObj in operationObj.parameters:
                    print 'type of parameter', paramObj.name, 'of operation', operationObj.name, \
                          'of class', classObj.name, '=', paramObj.type
    print ''
    
    # testing for number of parameter
    print '##testing for number of parameter##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            for operationObj in classObj.operations:
                print 'number of parameter operation', operationObj.name, \
                      'of class', classObj.name, '=', len(operationObj.parameters)
    print ''
    
    # testing for number of operation
    print '##testing for number of operation##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            print 'number of operation of', className, '=', classObj.number_of_operation()
    print ''
    
    # testing for class name
    print '##testing for class name##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            print 'class fullname =', classObj.fullname
            print 'class shortname =', classObj.name
    print ''

    # testing for number_of_inherited_operation_in_class
    print '##testing for number_of_inherited_operation_in_class##'
    className = 'CAMSEdit::SelectionSaver'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            print 'num of inheri. operations of', className, '= ', \
                  packageObj.number_of_inherited_operation_in_class(className)
    print ''
    
    # testing for operation scope
    print '##testing for operation scope##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            for operationObj in classObj.operations:
                print 'scope of operation', operationObj.name, 'of class', className, \
                      '=', operationObj.visibility
    print ''
    
    # testing for operation owner
    print '##testing for operation owner##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            for operationObj in classObj.operations:
                print 'owner of operation', operationObj.name, 'of class', className, \
                      '=', operationObj.owner.fullname
    print ''
    
    # testing for virtual declaration
    print '##testing for virtual declaration##'
    className = 'unitpp::tester'
    for packageObj in projectObj.packages:
        if className in packageObj.classes.keys():
            classObj = packageObj.classes[className]
            for operationObj in classObj.operations:
                print 'virtual of operation', operationObj.name, 'of class', className, \
                      '=', operationObj.isVirtual
    print ''
'''