__Version__ = "0.1"
__Owner__ = "jopwil"
__Update_Date__ = "2006/09/17"

def Sum(pro, Metric):
	"""Calculate the sum of class-level metrics' values within a project"""
	summation = 0.0	
	rawData = []
	classList = []
	returnValue = {}
	for packageObj in pro.packages:		
		for dd in packageObj.classes.keys():
			classList.append(dd)
			classObj = packageObj.classes[dd]
			mValue = Metric(classObj,pro)
			rawData.append(mValue)
			if mValue == None:
				pass
			else:
				summation = summation + mValue
	returnValue['metric'] = Metric.__doc__
	returnValue['sum'] = summation
	returnValue['values'] = rawData
	returnValue['classes'] = classList
	
	return returnValue

def Mean(pro, Metric):
	"""Calculate a mean of class-level metrics' values within a project"""
	count = 0.0
	average = 0.0
	summation = 0.0
	rawData = []
	classList = []
	returnValue = {}
	for packageObj in pro.packages:		
		for dd in packageObj.classes.keys():
			classList.append(dd)
			classObj = packageObj.classes[dd]
			mValue = Metric(classObj,pro)
			rawData.append(mValue)
			if mValue == None:
				summation = summation 
			else:
				summation = summation + mValue
				count = count + 1.0
	
	# [seLain:] count might be zero, but I don't now what this is ...
	if count != 0:
		average = summation / count
	else:
		average = 0
	
	
	returnValue['metric'] = Metric.__doc__
	returnValue['average'] = average
	returnValue['values'] = rawData
	returnValue['classes'] = classList
	return returnValue


#  ------------------------------------------------------------ Class-level metrics


def NOP_C_Metric(aClass, *info):
	"""QMOOD suite's NOP: Number of Polymorphic"""
	allOperations = aClass.operations
	accNOP = 0.0
	for operation in allOperations:
		if operation.isVirtual:
			accNOP = accNOP + 1.0
	return accNOP

def CIS_C_Metric(aClass, *info):
	"""QMOOD suite's Class Interface Size"""
	allOperations = aClass.operations
	accCIS = 0.0
	for operation in allOperations:
		if operation.visibility == 'public':
			accCIS = accCIS + 1.0
	return accCIS
	
def NOM_C_Metric(aClass, *info):
	"""QMOOD suite's NOM: Number of Methods"""
	accAON = 0.0
	allOperations = aClass.operations
	accAON = len(allOperations)
	return accAON

def DSC_C_Metric(aClass, *info):
	"""QMOOD suite's DSC: Design Size in Classes"""
	return 1.0

def DAM_C_Metric(aClass, *info):
	"""QMOOD suite's DAM: Data Access Metric"""
	if aClass.number_of_attribute() != 0:
		accDAM = float(aClass.number_of_private_attribute())/float(aClass.number_of_attribute())
	else:
		accDAM = 0.0
	return accDAM
def DCC_C_Metric(aClass, *info):
	"""QMOOD suite's DCC: Direct Class Coupling"""
	allAttributes = aClass.attributes
	types = []
	primitiveTypes = ['unsigned char', 'signed char','char', 'short', 'short int' ,'unsigned short int', 'int', 'unsigned int', 'signed int', 'long', 'long int','unsigned long int', 'signed long int', 'bool','float','double', 'long double', 'wchar_t']
	for attribute in allAttributes:
		if attribute.type not in types:
			types.append(attribute.type)
	
	allOperations = aClass.operations
	for operation in allOperations:
		allParameters = operation.parameters
		for parameter in allParameters:
			if parameter.type not in types:
				types.append(parameter.type)
	for type in types:
		if type in primitiveTypes:
			types.remove(type)
	#print types
	return len(types)

def CAM_C_Metric(aClass, *info):
    """QMOOD suite's CAM: Cohesion Among Methods of Class"""
    allOperations = aClass.operations
    accParameters = 0.0
    for operation in allOperations:
        accParameters = accParameters + len(operation.parameters)
    setParameterTypes = []
    for operation in allOperations:
        for parameter in operation.parameters:
            if parameter.type not in setParameterTypes:
                setParameterTypes.append(parameter.type)
    div = float(len(allOperations) * len(setParameterTypes))
    if div == 0:
        numCAM = 0.0
    else:
        numCAM = float((accParameters)/div)
    #print numCAM
    return numCAM

def MOA_C_Metric(aClass, *info):
    """QMOOD suite's MOA: Measure of Aggregation"""
    allAttributes = aClass.attributes
    accMOA = 0.0
    allClasses = []
    for packageObj in info[0].packages:
        for aclass in packageObj.classes.keys():
            allClasses.append(aclass)
    for attribute in allAttributes:
        for classname in allClasses:
            if classname.split('::')[-1] == attribute.type :
                accMOA = accMOA + 1
                break
    return accMOA

	
def ION_C_Metric(aClass, *info):
	aclass = aClass.name
	accION = 0.0
	for packageObj in info[0].packages:
		if aclass in packageObj.classes.keys():
			accION = packageObj.number_of_inherited_operation_in_class(aclass)
			
	print aclass, " : ", accION
	return accION
	
def MFA_C_Metric(aClass, *info):
	"""QMOOD suite's DSC: Measure of Functional Abstraction"""
	allOperations = aClass.operations
	accMFA = 0
	for operation in allOperations:
		if FCV_M_Metric(operation, aClass) :
			accMFA = accMFA + 1
	
	return accMFA
		
#  ------------------------------------------------------------ Project-level metrics

def DSC_P_Metric(projectObj):
	"""QMOOD suite's DSC: Design Size in Classes"""
	acc = 0.0
	for packageObj in projectObj.packages:
		for aclass in packageObj.classes.keys():
			acc = acc + 1
	return acc

def NOH_P_Metric(projectObj):
	"""QMOOD suite's NOH : Number of Hierarchies"""
	return projectObj.number_of_inheritence()
	
def ANA_P_Metric(projectObj): 
	"""QMOOD suite's ANA: Average Number of Ancestors"""
	acc = 0.0
	
	# [seLain:] count might be zero, but I don't now what this is ...
	if NOH_P_Metric(projectObj) != 0:
		acc = (float(projectObj.number_of_internalBaseClass()))/float(NOH_P_Metric(projectObj))
	else:
		acc = 0.0
		
	return acc
	

#  ------------------------------------------------------------ Method-level metrics

def FCV_M_Metric(aFeature, aClass):
	allFeatures = aClass.operations
	result = False
	if aFeature in allFeatures:
		if (((aFeature.owner.fullname == aClass.name) or (aFeature.visibility == 'public') or ((aFeature.visibility == 'protected') and (FCV_utility(aFeature, aClass)))) or (( aClass.number_of_operation() == 0) and (aFeature.visibility == 'public'))):
			result = True
	return result
			

def FCV_utility(aFeature, aClass):
	allFeaturesOfParentClass = []
	result = True
	for eachClass in aClass.parents:
		for eachFeature in eachClass:
			allFeaturesOfParentClass.append(eachFeature)
	if aFeature in allFeaturesOfParentClass:
		result = True
	else:
		result = False
	return result
	
